@@ -15,8 +15,7 @@ limitations under the License.
 """
 from __future__ import annotations
 
-import abc
-from typing import TYPE_CHECKING, Iterable
+from typing import TYPE_CHECKING, Iterable, Optional
 
 if TYPE_CHECKING:
     from cvxpy import Constant, Parameter, Variable
@@ -29,6 +28,7 @@ import numpy.linalg as LA
 import scipy.sparse as sp
 
 import cvxpy.interface as intf
+import cvxpy.settings as s
 from cvxpy.constraints.constraint import Constraint
 from cvxpy.expressions import expression
 from cvxpy.settings import (
@@ -53,10 +53,8 @@ class Leaf(expression.Expression):
 
     Parameters
     ----------
-    shape : Iterable of ints or int
-        The leaf dimensions. Either an integer n for a 1D shape, or an
-        iterable where the semantics are the same as NumPy ndarray shapes.
-        **Shapes cannot be more than 2D**.
+    shape : int or tuple of ints
+        Shape of the leaf, e.g., ``(3, 2)`` or ``2``.
     value : numeric type
         A value to assign to the leaf.
     nonneg : bool
@@ -75,17 +73,17 @@ class Leaf(expression.Expression):
         Is the variable constrained to be negative semidefinite?
     Hermitian : bool
         Is the variable Hermitian?
-    boolean : bool or list of tuple
+    boolean : bool or Iterable
         Is the variable boolean? True, which constrains
         the entire Variable to be boolean, False, or a list of
         indices which should be constrained as boolean, where each
         index is a tuple of length exactly equal to the
         length of shape.
-    integer : bool or list of tuple
+    integer : bool or Iterable
         Is the variable integer? The semantics are the same as the
         boolean argument.
-    sparsity : list of tuplewith
-        Fixed sparsity pattern for the variable.
+    sparsity : Iterable
+        Is the variable sparse?
     pos : bool
         Is the variable positive?
     neg : bool
@@ -94,19 +92,18 @@ class Leaf(expression.Expression):
         An iterable of length two specifying lower and upper bounds.
     """
 
-    __metaclass__ = abc.ABCMeta
-
     def __init__(
-        self, shape: int | Iterable[int, ...], value=None, nonneg: bool = False,
+        self, shape: int | tuple[int, ...], value = None, nonneg: bool = False,
         nonpos: bool = False, complex: bool = False, imag: bool = False,
         symmetric: bool = False, diag: bool = False, PSD: bool = False,
         NSD: bool = False, hermitian: bool = False,
-        boolean: bool = False, integer: bool = False,
-        sparsity=None, pos: bool = False, neg: bool = False, bounds: Iterable | None=None
+        boolean: Iterable | bool = False, integer: Iterable | bool = False,
+        sparsity: Iterable | bool = False, pos: bool = False, neg: bool = False,
+        bounds: Iterable | None = None
     ) -> None:
         if isinstance(shape, numbers.Integral):
             shape = (int(shape),)
-        elif len(shape) > 2:
+        elif not s.ALLOW_ND_EXPR and len(shape) > 2:
             raise ValueError("Expressions of dimension greater than 2 "
                              "are not supported.")
         for d in shape:
@@ -126,21 +123,28 @@ class Leaf(expression.Expression):
                            'complex': complex, 'imag': imag,
                            'symmetric': symmetric, 'diag': diag,
                            'PSD': PSD, 'NSD': NSD,
-                           'hermitian': hermitian, 'boolean': bool(boolean),
+                           'hermitian': hermitian, 'boolean': boolean,
                            'integer':  integer, 'sparsity': sparsity, 'bounds': bounds}
-
-        if boolean:
-            self.boolean_idx = boolean if not isinstance(boolean, bool) else list(
-                np.ndindex(max(shape, (1,))))
-        else:
+        if boolean is True:
+            shape = max(shape, (1,))
+            flat_idx = np.arange(np.prod(shape))
+            self.boolean_idx = np.unravel_index(flat_idx, shape, order='F')
+        elif boolean is False:
             self.boolean_idx = []
-
-        if integer:
-            self.integer_idx = integer if not isinstance(integer, bool) else list(
-                np.ndindex(max(shape, (1,))))
         else:
+            self.boolean_idx = boolean
+        if integer is True:
+            shape = max(shape, (1,))
+            flat_idx = np.arange(np.prod(shape))
+            self.integer_idx = np.unravel_index(flat_idx, shape, order='F')
+        elif integer is False:
             self.integer_idx = []
-
+        else:
+            self.integer_idx = integer
+        if sparsity:
+            self.sparse_idx = self._validate_indices(sparsity)
+        else:
+            self.sparse_idx = []
         # Only one attribute be True (except can be boolean and integer).
         true_attr = sum(1 for k, v in self.attributes.items() if v)
         # HACK we should remove this feature or allow multiple attributes in general.
@@ -149,17 +153,35 @@ class Leaf(expression.Expression):
         if true_attr > 1:
             raise ValueError("Cannot set more than one special attribute in %s."
                              % self.__class__.__name__)
-
         if value is not None:
             self.value = value
 
         self.args = []
-
         self.bounds = bounds
 
-    def _get_attr_str(self) -> str:
-        """Get a string representing the attributes.
+    def _validate_indices(self, indices: list[tuple[int]] | tuple[np.ndarray]) -> None:
         """
+        Validate the sparsity pattern for a leaf node.
+    
+        Parameters:
+        indices: List or tuple of indices indicating the positions of non-zero elements.
+        """
+        if not all(len(idx) == len(indices[0]) for idx in indices):
+            raise ValueError("All index tuples in indices must have the same length.")
+        
+        if len(indices) != len(self._shape):
+            raise ValueError(f"Indices should have {len(self._shape)} dimensions.")
+        
+        # convert list/tuple to array to validate indices within bounds
+        indices = np.array(indices)
+
+        if np.any(indices < 0) or np.any(indices >= np.array(self._shape).reshape(-1, 1)):
+            raise ValueError(
+                f"Indices are out of bounds for expression with shape {self._shape}.")
+        return tuple(indices)
+    
+    def _get_attr_str(self) -> str:
+        """Get a string representing the attributes."""
         attr_str = ""
         for attr, val in self.attributes.items():
             if attr != 'real' and val:
@@ -187,91 +209,74 @@ class Leaf(expression.Expression):
         return self  # Leaves are not deep copied.
 
     def get_data(self) -> None:
-        """Leaves are not copied.
-        """
+        """Leaves are not copied."""
 
     @property
     def shape(self) -> tuple[int, ...]:
-        """ tuple : The dimensions of the expression.
-        """
+        """The dimensions of the expression."""
         return self._shape
 
     def variables(self) -> list[Variable]:
-        """Default is empty list of Variables.
-        """
+        """Default is empty list of Variables."""
         return []
 
     def parameters(self) -> list[Parameter]:
-        """Default is empty list of Parameters.
-        """
+        """Default is empty list of Parameters."""
         return []
 
     def constants(self) -> list[Constant]:
-        """Default is empty list of Constants.
-        """
+        """Default is empty list of Constants."""
         return []
 
     def is_convex(self) -> bool:
-        """Is the expression convex?
-        """
+        """Is the expression convex?"""
         return True
 
     def is_concave(self) -> bool:
-        """Is the expression concave?
-        """
+        """Is the expression concave?"""
         return True
 
     def is_log_log_convex(self) -> bool:
-        """Is the expression log-log convex?
-        """
+        """Is the expression log-log convex?"""
         return self.is_pos()
 
     def is_log_log_concave(self) -> bool:
-        """Is the expression log-log concave?
-        """
+        """Is the expression log-log concave?"""
         return self.is_pos()
 
     def is_nonneg(self) -> bool:
-        """Is the expression nonnegative?
-        """
+        """Is the expression nonnegative?"""
         return (self.attributes['nonneg'] or self.attributes['pos'] or
                 self.attributes['boolean'])
 
     def is_nonpos(self) -> bool:
-        """Is the expression nonpositive?
-        """
+        """Is the expression nonpositive?"""
         return self.attributes['nonpos'] or self.attributes['neg']
 
     def is_pos(self) -> bool:
-        """Is the expression positive?
-        """
+        """Is the expression positive?"""
         return self.attributes['pos']
 
     def is_neg(self) -> bool:
-        """Is the expression negative?
-        """
+        """Is the expression negative?"""
         return self.attributes['neg']
 
     def is_hermitian(self) -> bool:
-        """Is the Leaf hermitian?
-        """
+        """Is the Leaf hermitian?"""
         return (self.is_real() and self.is_symmetric()) or \
             self.attributes['hermitian'] or self.is_psd() or self.is_nsd()
 
     def is_symmetric(self) -> bool:
-        """Is the Leaf symmetric?
-        """
+        """Is the Leaf symmetric?"""
         return self.is_scalar() or \
             any(self.attributes[key] for key in ['diag', 'symmetric', 'PSD', 'NSD'])
 
     def is_imag(self) -> bool:
-        """Is the Leaf imaginary?
-        """
+        """Is the Leaf imaginary?"""
         return self.attributes['imag']
 
     def is_complex(self) -> bool:
-        """Is the Leaf complex valued?
-        """
+        """Is the Leaf complex valued?"""
         return self.attributes['complex'] or self.is_imag() or self.attributes['hermitian']
 
     def _has_lower_bounds(self) -> bool:
@@ -418,6 +423,10 @@ class Leaf(expression.Expression):
                     return val
                 w[bad] = 0
             return (V * w).dot(V.T)
+        elif self.attributes['sparsity']:
+            new_val = np.zeros(self.shape)
+            new_val[self.sparse_idx] = val
+            return new_val
         else:
             return val
 
@@ -426,9 +435,8 @@ class Leaf(expression.Expression):
         self._value = val
 
     @property
-    def value(self):
-        """NumPy.ndarray or None: The numeric value of the parameter.
-        """
+    def value(self) -> Optional[np.ndarray]:
+        """The numeric value of the expression."""
         return self._value
 
     @value.setter
@@ -513,33 +521,27 @@ class Leaf(expression.Expression):
         return val
 
     def is_psd(self) -> bool:
-        """Is the expression a positive semidefinite matrix?
-        """
+        """Is the expression a positive semidefinite matrix?"""
         return self.attributes['PSD']
 
     def is_nsd(self) -> bool:
-        """Is the expression a negative semidefinite matrix?
-        """
+        """Is the expression a negative semidefinite matrix?"""
         return self.attributes['NSD']
 
     def is_diag(self) -> bool:
-        """Is the expression a diagonal matrix?
-        """
+        """Is the expression a diagonal matrix?"""
         return self.attributes['diag']
 
     def is_quadratic(self) -> bool:
-        """Leaf nodes are always quadratic.
-        """
+        """Leaf nodes are always quadratic."""
         return True
 
     def has_quadratic_term(self) -> bool:
-        """Leaf nodes are not quadratic terms.
-        """
+        """Leaf nodes are not quadratic terms."""
         return False
 
     def is_pwl(self) -> bool:
-        """Leaf nodes are always piecewise linear.
-        """
+        """Leaf nodes are always piecewise linear."""
         return True
 
     def is_dpp(self, context: str = 'dcp') -> bool:
